@@ -108,7 +108,7 @@
 -record(editor, { port :: port(), file :: file:name(), requester :: pid() }).
 -record(state, { tty :: prim_tty:state() | undefined,
                  write :: reference() | undefined,
-                 read :: reference() | undefined,
+                 read :: reference() | eof | undefined,
                  terminal_mode :: raw | cooked | disabled,
                  shell_started = new :: new | old | false,
                  editor :: #editor{} | undefined,
@@ -473,7 +473,7 @@ server(info, {ReadHandle,{data,UTF8Binary}}, State = #state{ read = ReadHandle }
     end;
 server(info, {ReadHandle,eof}, State = #state{ read = ReadHandle }) ->
     State#state.current_group ! {self(), eof},
-    {keep_state, State#state{ read = undefined }};
+    {keep_state, State#state{ read = eof }};
 server(info,{ReadHandle,{signal,Signal}}, State = #state{ tty = TTYState, read = ReadHandle }) ->
     {keep_state, State#state{ tty = prim_tty:handle_signal(TTYState, Signal) }};
 
@@ -608,10 +608,25 @@ server(info,{'EXIT', Group, Reason}, State) ->
                                                       current_group = NewGroup,
                                                       groups = Gr2 }};
                         _ -> % remote shell
-                            NewTTYState = io_requests(
-                                            Reqs ++ [{put_chars,unicode,<<"(^G to start new job) ***\n">>}],
-                                            State#state.tty),
-                            {keep_state, State#state{ tty = NewTTYState, groups = Gr1 }}
+                            %% If the readhandle has terminated, then we should quit
+                            case State#state.read =:= eof of
+                                true ->
+                                    NewTTYState = io_requests(Reqs,
+                                                State#state.tty),
+                                    _ = io_request({put_chars_sync,unicode,<<"Read EOF ***\n">>, {self(), none}}, NewTTYState),
+                                    WriterRef = State#state.write,
+                                    receive
+                                        {WriterRef, ok} -> ok
+                                    after 100 ->
+                                        ok
+                                    end,
+                                    erlang:halt(0, []);
+                                false ->
+                                    NewTTYState = io_requests(
+                                                    Reqs ++ [{put_chars,unicode,<<"(^G to start new job) ***\n">>}],
+                                                    State#state.tty),
+                                    {keep_state, State#state{ tty = NewTTYState, groups = Gr1 }}
+                            end
                     end;
                 _ ->
                     {keep_state, State#state{ groups = gr_del_pid(State#state.groups, Group) }}
@@ -807,7 +822,8 @@ switch_cmd({r,Node,Shell}, Gr0, Dumb) when is_atom(Node), is_atom(Shell) ->
                     Gr = gr_add_cur(Gr0, Pid, {Node,Shell,start,[]}),
                     {retry, [], Gr};
                 false ->
-                    {retry, [{put_chars,unicode,<<"Could not connect to node\n">>}]}
+                    Bin = atom_to_binary(Node),
+                    {retry, [{put_chars,unicode,<<"Could not connect to node ", Bin/binary, "\n">>}]}
             end;
         false ->
             {retry, [{put_chars,unicode,"Node is not alive\n"}]}
